@@ -2,7 +2,7 @@ import bitmapist
 
 
 def test_delta_hour():
-    ev = bitmapist.HourEvents('foo', 2014, 1, 1, 0)
+    ev = bitmapist.HourEvents("foo", 2014, 1, 1, 0)
     n = ev.next()
     assert (n.year, n.month, n.day, n.hour) == (2014, 1, 1, 1)
     p = ev.prev()
@@ -10,7 +10,7 @@ def test_delta_hour():
 
 
 def test_delta_day():
-    ev = bitmapist.DayEvents('foo', 2014, 1, 1)
+    ev = bitmapist.DayEvents("foo", 2014, 1, 1)
     n = ev.next()
     assert (n.year, n.month, n.day) == (2014, 1, 2)
     p = ev.prev()
@@ -18,7 +18,7 @@ def test_delta_day():
 
 
 def test_delta_week():
-    ev = bitmapist.WeekEvents('foo', 2014, 1)
+    ev = bitmapist.WeekEvents("foo", 2014, 1)
     n = ev.next()
     assert (n.year, n.week) == (2014, 2)
     p = ev.prev()
@@ -26,7 +26,7 @@ def test_delta_week():
 
 
 def test_delta_month():
-    ev = bitmapist.MonthEvents('foo', 2014, 1)
+    ev = bitmapist.MonthEvents("foo", 2014, 1)
     n = ev.next()
     assert (n.year, n.month) == (2014, 2)
     p = ev.prev()
@@ -34,7 +34,7 @@ def test_delta_month():
 
 
 def test_delta_year():
-    ev = bitmapist.YearEvents('foo', 2014)
+    ev = bitmapist.YearEvents("foo", 2014)
     n = ev.next()
     assert n.year == 2015
     p = ev.prev()
